@@ -4,6 +4,7 @@ import json
 import re
 import random
 import argparse
+import sys
 from attrdict import AttrDict
 import attrdict
 from getpass import getpass
@@ -12,7 +13,14 @@ from getpass import getpass
 
 
 class Bot:
-    def __init__(self, nick, room = "bots", owner = None, password = "", help = None, ping = None, important = False, killed = None, API_timeout = 10):
+    ERROR = "\033[1;31;40m"
+    NORMAL = "\033[m"
+
+    def log(self, message, error = False):
+        print((ERROR if error else NORMAL) + message)
+        sys.stdout.flush()
+
+    def __init__(self, nick, room = "bots", owner = None, password = "", help = None, ping = None, important = False, killed = None, API_timeout = 10, advanced = False, function = None):
         parser = argparse.ArgumentParser(description=f'{nick}: A euphoria.io bot.')
         parser.add_argument("--test", "--debug", "-t", help = "Used to debug dev builds. Sends bot to &test instead of its default room.", action = 'store_true')
         parser.add_argument("--room", "-r", help = f"Set the room the bot will be placed in. Default: {room}", action = "store", default = room)
@@ -22,7 +30,7 @@ class Bot:
 
         self.nick = nick
         self.room = args.room if args.test != True else "test"
-        print("Debug: " + str(args.test))
+        self.log("Debug: " + str(args.test))
         self.normname = re.sub(r"\s+", "", self.nick)
         self.owner = owner if owner != None else "nobody"
         self.normowner = re.sub(r"\s+", "", self.owner)
@@ -33,6 +41,8 @@ class Bot:
         self.important = important
         self.killed = killed
         self.API_timeout = API_timeout
+        self.advanced = advanced
+        self.function = function
 
 
     def connect(self):
@@ -47,8 +57,8 @@ class Bot:
             self._handle_auth(self.password)
             self.set_nick(self.nick)
         else:
-            print(reply)
-        print(f'connected to &{self.room}.')
+            self.log(reply, error = True)
+        self.log(f'connected to &{self.room}.')
 
     def send_msg(self, msgString, parent = None):
         if re.search(r"^\[.+,.+\]$", msgString):
@@ -62,10 +72,10 @@ class Bot:
                 msg = AttrDict(json.loads(msgJSON))
                 if msg.type == "send-reply":
                     reply = msg
-                    print(f'Message sent: {reply} replying to: {parent.data.id} by {parent.data.sender.name}')
+                    self.log(f'Message sent: {reply} replying to: {parent.data.id} by {parent.data.sender.name}')
                     return reply
                 if i > self.API_timeout:
-                    print("send-reply API response not recorded. reason: too many events before send-reply.")
+                    self.log("send-reply API response not recorded. reason: too many events before send-reply.", True)
                     return None
 
         elif isinstance(parent, str):
@@ -77,14 +87,14 @@ class Bot:
                 msg = AttrDict(json.loads(msgJSON))
                 if msg.type == "send-reply":
                     reply = msg
-                    print(f'Message sent: {reply} replying to: {parent}')
+                    self.log(f'Message sent: {reply} replying to: {parent}')
                     return reply
                 if i > self.API_timeout:
-                    print("send-reply API response not recorded. reason: too many events before send-reply.")
+                    self.log("send-reply API response not recorded. reason: too many events before send-reply.", True)
                     return None
 
         else:
-            raise BadMessageError(f'message not sent, because type of parent was {type(parent)}').with_traceback(f'parent = {parent}')
+            raise BadMessageError(f'message not sent, because type of parent was {type(parent)}').with_traceback(f'parent = {parent}', error = True)
 
 
 
@@ -95,26 +105,55 @@ class Bot:
         self.start()
         self.send_msg("Restarted", parent = msg)
 
-    def simple_start(self):
-        try:
-            for msgJSON in self.conn:
-                msg = AttrDict(json.loads(msgJSON))
-                if msg.type == 'ping-event':
-                    self._handle_ping(msg)
-                elif msg.type == 'send-event' and msg.data.sender.name != self.nick:
-                    self._handle_message(msg)
-                elif msg.type == 'error':
-                    print(msg)
-                elif msg.type == 'bounce-event':
-                    self._handle_auth(msg)
-                else:
-                    self._handle_other(msg)
-        except KilledError:
-            pass
-
     def nothing(msg):
         return
 
+    def start(self, function = None, advanced = False):
+        if not (self.advanced or advanced):
+            try:
+                for msgJSON in self.conn:
+                    msg = AttrDict(json.loads(msgJSON))
+                    if msg.type == 'ping-event':
+                        self._handle_ping(msg)
+                    elif msg.type == 'send-event' and msg.data.sender.name != self.nick:
+                        self._handle_message(msg)
+                    elif msg.type == 'error':
+                        self.log(msg, error = True)
+                    elif msg.type == 'bounce-event':
+                        self._handle_auth(msg)
+                    else:
+                        self._handle_other(msg)
+            except KilledError:
+                pass
+        else:
+            if self.function == None and function != None and callable(function):
+                self.function = function
+            else:
+                self.log("No function passed to advanced bot's start function.")
+            try:
+                for msgJSON in self.conn:
+                    msg = AttrDict(json.loads(msgJSON))
+                    if msg.type == 'send-event' and msg.data.sender.name != self.nick:
+                        if re.search(f'^!kill @{self.normname}$', msg.data.content) != None:
+                            self._handle_kill(msg)
+                        if re.search(f'^!forcekill @{self.normname}$', msg.data.content) != None:
+                            self.kill()
+                        if re.search(f'^!restart @{self.normname}$', msg.data.content) != None:
+                            self.restart()
+                    if msg.type in self.handlers.keys():
+                        self.handlers[msg.type](msg)
+                    else:
+                        if function != None and callable(function):
+                            function(msg)
+                        elif callable(self.function):
+                            self.function(msg)
+                        else:
+                            self.log("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True)
+            except KilledError:
+                pass
+
+
+
     def _is_privileged(self, user):
         if "is_manager" in user.keys() or user.name == self.owner:
             return True
@@ -141,27 +180,6 @@ class Bot:
             else:
                 self.killed = "/me has died."
 
-    def advanced_start(self, function = nothing):
-        if callable(function):
-            try:
-                for msgJSON in self.conn:
-                    msg = AttrDict(json.loads(msgJSON))
-                    if msg.type == 'send-event' and msg.data.sender.name != self.nick:
-                        if re.search(f'^!kill @{self.normname}$', msg.data.content) != None:
-                            self._handle_kill(msg)
-                        if re.search(f'^!forcekill @{self.normname}$', msg.data.content) != None:
-                            self.kill()
-                        if re.search(f'^!restart @{self.normname}$', msg.data.content) != None:
-                            self.restart()
-                    if msg.type in self.handlers.keys():
-                        self.handlers[msg.type](msg)
-                    else:
-                        self.function(msg)
-            except KilledError:
-                pass
-        else:
-            print("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.")
-
     def _handle_ping(self, msg):
         self.conn.send(json.dumps({'type': 'ping-reply', 'data': {'time': msg.data.time}}))
 
@@ -172,9 +190,9 @@ class Bot:
             self.kill()
         if re.search(f'^!restart @{self.normname}$', msg.data.content) != None:
             self.restart()
-        if re.search('^!ping$', msg.data.content) != None or re.search(f'^!ping @{self.normname}$', msg.data.content) != None:
+        if re.search('^!ping$', msg.data.content) != None or re.search(f'^!ping @(?i){self.normname}$', msg.data.content) != None:
             self.send_msg(self.ping, parent = msg)
-        if re.search(f'^!help @{self.normname}$', msg.data.content) != None:
+        if re.search(f'^!help @(?i){self.normname}$', msg.data.content) != None:
             self.send_msg(self.help, parent = msg)
         for regex, response in self._regexes.items():
             if re.search(regex, msg.data.content) != None:
@@ -211,13 +229,13 @@ class Bot:
             if msg.type == "auth-reply":
                 reply = msg
                 if reply.data.success == True:
-                    print(f'Successfully logged into {self.room}.')
+                    self.log(f'Successfully logged into {self.room}.')
                 else:
-                    print(f'Login unsuccessful. Reason: {reply.data.reason}')
+                    self.log(f'Login unsuccessful. Reason: {reply.data.reason}', error = True)
                     self._handle_auth(getpass("Enter the correct password: "))
                 break
             if i > self.API_timeout:
-                print("Auth Error: auth-reply API response not recorded.")
+                self.log("Auth Error: auth-reply API response not recorded.", error = True)
                 break
 
     def _handle_other(self, msg):
@@ -243,7 +261,7 @@ class Bot:
                 reply = msg
                 return reply.data.listing
             if i > self.API_timeout:
-                print("who-reply API response not recorded. reason: too many events before who-reply.")
+                self.log("who-reply API response not recorded. reason: too many events before who-reply.", error = True)
                 return None
 
 
@@ -256,7 +274,7 @@ class Bot:
         if callable(function):
             self.handlers += {eventString : function}
         else:
-            print(f"WARNING: handler for {eventString} not callable, handler not set.")
+            self.log(f"WARNING: handler for {eventString} not callable, handler not set.", error = True)
 
 class BotError(Exception):
     pass
@@ -264,9 +282,9 @@ class BotError(Exception):
 class KilledError(BotError):
     def __init__(self, message = "bot killed."):
         self.message = message
-        print(f'KilledError: {message}')
+        self.log(f'KilledError: {message}', error = True)
 
 class BadMessageError(BotError):
     def __init__(self, message):
         self.message = message
-        print(f'BadMessageError: {message}')
+        self.log(f'BadMessageError: {message}', error = True)
